package question15

import (
	"os"
	"strconv"
	"testing"
)

const DEFAULT_SIZE = 3

var SIZE = DEFAULT_SIZE

func TestMain(m *testing.M) {
	if len(os.Args) > 1 {
		if n, err := strconv.Atoi(os.Args[1]); err != nil {
			if len(os.Args) > 2 {
				if n, err := strconv.Atoi(os.Args[2]); err != nil {
					SIZE = DEFAULT_SIZE
				} else {
					SIZE = n
				}
			} else {
				SIZE = DEFAULT_SIZE
			}
		} else {
			SIZE = n
		}
	}
	os.Exit(m.Run())
}

func BuildRegularTree(depth int, key *int) (t *Tree) {
	t = nil
	if depth >= 0 {
		k := *key
		*key += 1
		left := BuildRegularTree(depth-1, key)
		right := BuildRegularTree(depth-1, key)
		t = &Tree{k, left, right}
	}
	return
}

func logTree(tree *Tree, t *testing.T) {
	if tree != nil {
		t.Log("Tree {", tree.Key, ",")
		logTree(tree.Left, t)
		t.Log(",")
		logTree(tree.Right, t)
		t.Log(")")
	} else {
		t.Log(tree)
	}
}

func TestTrivialTree0(t *testing.T) {
	key := 1
	tree := BuildRegularTree(-1, &key)
	t.Log("Building regular tree of depth -1")
	l := tree.PreOrderList()
	t.Log("tree.PreOrderList() = ", l)
	if len(l) != 0 {
		t.Fatal("Wrong length or content")
	}
}

func TestTrivialTree1(t *testing.T) {
	key := 1
	tree := BuildRegularTree(0, &key)
	t.Log("Building regular tree of depth 0")
	l := tree.PreOrderList()
	t.Log("tree.PreOrderList() = ", l)
	if len(l) != 1 && l[0] != 1 {
		t.Fatal("Wrong length or content")
	}
}

func TestFixedDepthRegularTree(t *testing.T) {
	key := 1
	tree := BuildRegularTree(SIZE, &key)
	t.Logf("Building regular tree of depth %v", SIZE)
	l := tree.PreOrderList()
	t.Log("tree.PreOrderList() = ", l)
	if len(l) < key-1 {
		t.Fatal("Wrong length")
	}
	i := 0
	for ; i < len(l) && l[i] == i+1; i++ {
	}
	if i < len(l) {
		t.Fatal("Wrong content")
	}
}
