package question16

import (
	"fmt"
	"log"
	"os"
	"testing"
)

/*
 * The type SpanTree store spanning tree and their extra edges.
 * It provides the following operations
 * func (st *SpanTree) AddTreeEdge(x, y int)
 * st.AddTreeEdge(x,y) add the edge (x,y) to the tree
 * func (st *SpanTree) AddBackEdge(x, y int)
 * st.AddBackEdge(x, y) add the back edge (x,y) to the tree
 * func (st *SpanTree) AddForwardEdge(x, y int)
 * st.AddForwardEdge(x, y) add the forward edge (x,y) to the tree
 * func (st *SpanTree) AddCrossEdge(x, y int)
 * st.AddCrossEdge(x, y) add the cross edge (x,y) to the tree
 */

type Edge struct {
	Src, Dst int
}

type SpanTree struct {
	Edges, BackEdges, ForwardEdges, CrossEdges []Edge
}

func NewSpanTree() *SpanTree {
	return &SpanTree{}
}

func (st *SpanTree) AddTreeEdge(x, y int) {
	st.Edges = append(st.Edges, Edge{x, y})
}

func (st *SpanTree) AddBackEdge(x, y int) {
	st.BackEdges = append(st.BackEdges, Edge{x, y})
}

func (st *SpanTree) AddForwardEdge(x, y int) {
	st.ForwardEdges = append(st.ForwardEdges, Edge{x, y})
}

func (st *SpanTree) AddCrossEdge(x, y int) {
	st.CrossEdges = append(st.CrossEdges, Edge{x, y})
}

func (st *SpanTree) LogTree(t *testing.T) {
	t.Log("Edges:", st.Edges)
	t.Log("BackEdges:", st.BackEdges)
	t.Log("ForwardEdges:", st.ForwardEdges)
	t.Log("CrossEdges:", st.CrossEdges)
}

func (st *SpanTree) BuildDotFile(fname string) {
	file, err := os.Create(fname)
	if err != nil {
		log.Println(err)
		return
	}
	defer file.Close()
	fmt.Fprintf(file, "digraph G {\n")
	for _, e := range st.Edges {
		fmt.Fprintf(file, "  %v -> %v;\n", e.Src, e.Dst)
	}
	for _, e := range st.BackEdges {
		fmt.Fprintf(file, "  %v -> %v [constraint=false color=blue];\n", e.Src, e.Dst)
	}
	for _, e := range st.ForwardEdges {
		fmt.Fprintf(file, "  %v -> %v [constraint=false color=green];\n", e.Src, e.Dst)
	}
	for _, e := range st.CrossEdges {
		fmt.Fprintf(file, "  %v -> %v [constraint=false color=red];\n", e.Src, e.Dst)
	}
	fmt.Fprintf(file, "}\n")
}
